from dual_quaternions import DualQuaternion
import numpy as np


class DualQuaternionSplines:
    def __init__(self, pose_vector, nr_interpolation_steps=10):
        self.pose_vector = pose_vector
        self.nr_interpolation_steps = nr_interpolation_steps
        self.dual_quaternions = self.parse_input_vector()

    def generate_trajectory(self):
        """Generate an interpolated trajectory from the list of dual quaternions."""
        interpolated_trajectory = []
        for i in range(len(self.dual_quaternions) - 1):
            interpolated_trajectory.extend(self.interpolate_dual_quaternions(self.dual_quaternions[i],
                                                                             self.dual_quaternions[i+1],
                                                                             self.nr_interpolation_steps))
        return interpolated_trajectory

    def parse_input_vector(self):
        """Parse the input vector into dual quaternions."""
        dual_quats = []
        for i in range(0, len(self.pose_vector), 7):
            pose = self.pose_vector[i:i+7]
            dq = self.quaternion_to_dual_quaternion(pose)
            dual_quats.append(dq)
        return dual_quats

    @staticmethod
    def quaternion_to_dual_quaternion(pose):
        """Convert position and quaternion to a dual quaternion.
        :param pose: [q_rw, q_rx, q_ry, q_rz, q_tx, q_ty, q_tz]"""
        return DualQuaternion.from_quat_pose_array(pose)

    @staticmethod
    def interpolate_dual_quaternions(dq1, dq2, steps=10):
        """Interpolate between two dual quaternions."""
        return [DualQuaternion.sclerp(dq1, dq2, t) for t in np.linspace(0, 1, steps)]





